/****************************************************************************
** Meta object code from reading C++ file 'game.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../bovo-26.04.2/src/game/game.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'game.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4bovo4GameE_t {};
} // unnamed namespace

template <> constexpr inline auto bovo::Game::qt_create_metaobjectdata<qt_meta_tag_ZN4bovo4GameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bovo::Game",
        "gameOver",
        "",
        "QList<Move>",
        "moves",
        "boardChanged",
        "Move",
        "move",
        "playerTurn",
        "oposerTurn",
        "replayBegin",
        "replayEnd",
        "undoAble",
        "undoNotAble",
        "replay",
        "undoLatest",
        "replayNext"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'gameOver'
        QtMocHelpers::SignalData<void(const QList<Move> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'boardChanged'
        QtMocHelpers::SignalData<void(const Move &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'playerTurn'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'oposerTurn'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replayBegin'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replayEnd'
        QtMocHelpers::SignalData<void(const QList<Move> &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'undoAble'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'undoNotAble'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'move'
        QtMocHelpers::SlotData<void(const Move &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'replay'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'undoLatest'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'replayNext'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Game, qt_meta_tag_ZN4bovo4GameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bovo::Game::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4bovo4GameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4bovo4GameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4bovo4GameE_t>.metaTypes,
    nullptr
} };

void bovo::Game::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Game *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->gameOver((*reinterpret_cast<std::add_pointer_t<QList<Move>>>(_a[1]))); break;
        case 1: _t->boardChanged((*reinterpret_cast<std::add_pointer_t<Move>>(_a[1]))); break;
        case 2: _t->playerTurn(); break;
        case 3: _t->oposerTurn(); break;
        case 4: _t->replayBegin(); break;
        case 5: _t->replayEnd((*reinterpret_cast<std::add_pointer_t<QList<Move>>>(_a[1]))); break;
        case 6: _t->undoAble(); break;
        case 7: _t->undoNotAble(); break;
        case 8: _t->move((*reinterpret_cast<std::add_pointer_t<Move>>(_a[1]))); break;
        case 9: _t->replay(); break;
        case 10: _t->undoLatest(); break;
        case 11: _t->replayNext(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Game::*)(const QList<Move> & )>(_a, &Game::gameOver, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)(const Move & )>(_a, &Game::boardChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)()>(_a, &Game::playerTurn, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)()>(_a, &Game::oposerTurn, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)()>(_a, &Game::replayBegin, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)(const QList<Move> & )>(_a, &Game::replayEnd, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)()>(_a, &Game::undoAble, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Game::*)()>(_a, &Game::undoNotAble, 7))
            return;
    }
}

const QMetaObject *bovo::Game::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bovo::Game::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4bovo4GameE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bovo::Game::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void bovo::Game::gameOver(const QList<Move> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void bovo::Game::boardChanged(const Move & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void bovo::Game::playerTurn()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void bovo::Game::oposerTurn()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void bovo::Game::replayBegin()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void bovo::Game::replayEnd(const QList<Move> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void bovo::Game::undoAble()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void bovo::Game::undoNotAble()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
